#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/timeb.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <ourhdr.h>


double TimeDiff( struct timeb Start, struct timeb Stop)
{
	double diff;

	if (Stop.millitm < Start.millitm) {
		Stop.millitm += 1000;
		Stop.time -= 1;
	}
	diff = (double) (Stop.time - Start.time) + (Stop.millitm - Start.millitm) / 1000.;
	return diff;
}


int main(int argc, char *argv[]) {
	long i, j, k, max, num_op = 50000000;
	double size;
	int **a,**b, **c;
	char err_msg[100];
	struct rusage usage;
	struct timeb TStart, TStop;

	if (argc < 2){
		fprintf( stderr, "Errore nei parametri. Occorre specificare la taglia della matrice\n");
		fprintf( stderr, "usage: %s size [#operations]\n", argv[0]);
		exit(-1);
	}

	size = atof(argv[1]);		// taglia acomplessiva in megabyte scelta dall'utente
	max = (int) sqrt( size / (sizeof(int) * 3.) * 1000000.);	// 3 matrici ognuna da max * max * 4 byte

	if (argc == 3)
		num_op = atoi(argv[2]);

	printf("Inizio allocazione di %.1fMb (3 matrici da %ld x %ld)\n", size, max, max);
	ftime( &TStart);
	// allocazione di tre matrici molto grandi
	// prima si allocano i puntatori alle righe
	if ((a = (int **) calloc( max, sizeof(int *))) == NULL)
		err_sys("matrice A");

	if ((b = (int **) calloc( max, sizeof(int *))) == NULL)
		err_sys("matrice B");

	if ((c = (int **) calloc( max, sizeof(int *))) == NULL)
		err_sys("matrice C");

	// poi si allocano una per una le righe intere
	for(i = 0; i < max; i++) {
		if ((a[i]=calloc(max, sizeof(int))) == NULL) {
			sprintf( err_msg, "Errore di allocazione matrice A, riga: %ld", i);
			err_sys( err_msg);
		}
		if ((b[i]=calloc(max, sizeof(int))) == NULL) {
			sprintf( err_msg, "Errore di allocazione matrice B, riga: %ld", i);
			err_sys( err_msg);
		}
		if ((c[i]=calloc(max, sizeof(int))) == NULL) {
			sprintf( err_msg, "Errore di allocazione matrice B, riga: %ld", i);
			err_sys( err_msg);
		}
	}

	printf("Fine allocazione. Inizio operazioni\n");


	for(i = 0; i < max; i++){
		for(j = 0; j < max; j++){
			b[i][j] = i + j;
		}
	}

	for(k = 0; k < num_op; k++){
		i = random() % max;
		j = random() % max;
		if (b[i][j] != 0)
			c[i][j] = a[j][i] / b[i][j] * random();
		if (k % 10000 == 0)
			putchar('.');
	}
	ftime( &TStop);
	printf("Fine operazioni\n");
	getrusage(RUSAGE_SELF, &usage );
	printf("taglia memoria residente  : %ld\n" , usage.ru_maxrss);
	printf("taglia shared mem size    : %ld\n" , usage.ru_ixrss);
	printf("taglia data segment virt. : %ld\n" , usage.ru_idrss);
	printf("taglia stack segment virt.: %ld\n" , usage.ru_isrss);
	printf("page fault  (major)       : %ld\n" , usage.ru_majflt);
	printf("page reclaim (minor)      : %ld\n" , usage.ru_minflt);
	printf("page swap (scambio)       : %ld\n" , usage.ru_nswap);
	printf("real time                 : %.3f\n", TimeDiff( TStart, TStop));
	printf("user CPU time             : %ld.%03ld\n",
			usage.ru_utime.tv_sec ,usage.ru_utime.tv_usec);
	printf("sys  CPU time             : %ld.%03ld\n",
			usage.ru_stime.tv_sec, usage.ru_stime.tv_usec);
	printf("context switch involontari: %ld\n" , usage.ru_nivcsw);
	printf("context switch volontari  : %ld\n" , usage.ru_nvcsw);
	printf("tempo nelle coda di wait  : %.3f\n",
			TimeDiff( TStart, TStop) - (double)
			(usage.ru_utime.tv_sec + usage.ru_utime.tv_usec / 1000000. +
			 usage.ru_stime.tv_sec + usage.ru_stime.tv_usec / 1000000.));
	return EXIT_SUCCESS;
}

